@@ -1,8 +1,10 @@
 from datetime import datetime
+
 from distutils.version import LooseVersion
-from redmine.utilities import to_string
-from redmine.managers import ResourceManager
-from redmine.exceptions import (
+
+from .utilities import to_string
+from .managers import ResourceManager
+from .exceptions import (
     ValidationError,
     ForbiddenError,
     ResourceAttrError,
@@ -573,6 +575,11 @@ class WikiPage(_Resource):
         return to_string(self.title)
 
     def __getattr__(self, item):
+        if item == 'parent' and item in self._attributes:
+            manager = ResourceManager(self.manager.redmine, 'WikiPage')
+            manager.params['project_id'] = self.manager.params.get('project_id', 0)
+            return manager.to_resource(self._attributes[item])
+
         # If a text attribute of a resource is missing, we should
         # refresh a resource automatically for user's convenience
         try:
